(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[141],{2931:(e,s,l)=>{"use strict";l.r(s),l.d(s,{default:()=>h});var r=l(5155),t=l(9545),a=l(7652),d=l(2960),c=l(6766);l(2115);var i=l(4615),n=l(7280),o=l(2954);let h=function(){let{darkMode:e,toggleDarkMode:s}=(0,n.D)(),{open:l}=(0,o.cL)(),{data:h,isLoading:m,error:u,refetch:x}=(0,d.I)({queryKey:["traces"],queryFn:a._K});return(0,r.jsxs)("div",{className:"flex flex-col h-full ".concat(l?"ml-0":"ml-18"),children:[(0,r.jsx)(i.Y,{darkMode:e,toggleDarkMode:s}),(0,r.jsx)("div",{className:"flex flex-col h-full",children:(0,r.jsx)("div",{className:"border rounded-lg flex flex-col bg-card/50 backdrop-blur-sm h-full pb-0",children:m?(0,r.jsx)("div",{className:"flex items-center justify-center h-full",children:(0,r.jsx)("div",{className:"animate-spin h-8 w-8 border-4 border-primary border-t-transparent rounded-full"})}):h&&h.length>0?(0,r.jsx)("div",{className:"flex-1 flex flex-col h-full",children:(0,r.jsx)(t.D,{data:h})}):(0,r.jsx)("div",{className:"text-center flex items-center justify-center h-full",children:(0,r.jsxs)("div",{children:[(0,r.jsx)(c.default,{src:"/pype-logo.png",alt:"PYPE Logo",width:100,height:100,className:"h-12 w-20 mx-auto mb-4"}),(0,r.jsx)("h3",{className:"text-lg font-medium",children:"No traces available"}),(0,r.jsx)("p",{className:"text-muted-foreground mt-2",children:"Trace data will appear here when you run experiments"})]})})})})]})}},4168:(e,s,l)=>{"use strict";l.d(s,{A:()=>r});var r=(0,l(6467).A)("outline","moon","IconMoon",[["path",{d:"M12 3c.132 0 .263 0 .393 0a7.5 7.5 0 0 0 7.92 12.446a9 9 0 1 1 -8.313 -12.454z",key:"svg-0"}]])},4615:(e,s,l)=>{"use strict";l.d(s,{Y:()=>n});var r=l(5155);l(2115);var t=l(8185),a=l(4168),d=l(285),c=l(6102),i=l(2954);function n(e){let{darkMode:s,toggleDarkMode:l,children:n}=e;return(0,r.jsx)("header",{className:"sticky top-0 z-50 border-b border-border/40 bg-card/50 backdrop-blur-sm w-full",children:(0,r.jsxs)("div",{className:"pr-6 pl-3 flex h-14 justify-between w-full items-center",children:[(0,r.jsxs)("div",{className:"flex items-center gap-3",children:[(0,r.jsx)(i.x2,{}),(0,r.jsx)("div",{className:"flex items-center gap-4",children:n})]}),(0,r.jsx)("div",{className:"flex items-center",children:(0,r.jsx)(c.Bc,{children:(0,r.jsxs)(c.m_,{children:[(0,r.jsx)(c.k$,{asChild:!0,children:(0,r.jsx)(d.$,{variant:"outline",size:"icon",className:"rounded-full",onClick:l,children:s?(0,r.jsx)(t.A,{className:"h-4 w-4"}):(0,r.jsx)(a.A,{className:"h-4 w-4"})})}),(0,r.jsx)(c.ZI,{children:(0,r.jsx)("p",{children:"Toggle theme"})})]})})})]})})}},5113:(e,s,l)=>{Promise.resolve().then(l.bind(l,2931))},7280:(e,s,l)=>{"use strict";l.d(s,{D:()=>c,N:()=>d});var r=l(5155),t=l(2115);let a=(0,t.createContext)({theme:"system",setTheme:()=>null,darkMode:!1,toggleDarkMode:()=>null});function d(e){let{children:s,defaultTheme:l="system",enableSystem:d=!0,attribute:c="data-theme"}=e,[i,n]=(0,t.useState)(l),[o,h]=(0,t.useState)("dark"===l);return(0,t.useEffect)(()=>{let e=window.document.documentElement;if(e.classList.remove("light","dark"),"system"===i&&d){let s=window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light";e.classList.add(s),h("dark"===s)}else e.classList.add(i),h("dark"===i)},[i,d]),(0,r.jsx)(a.Provider,{value:{theme:i,setTheme:n,darkMode:o,toggleDarkMode:()=>{n(o?"light":"dark")}},children:s})}let c=()=>{let e=(0,t.useContext)(a);if(void 0===e)throw Error("useTheme must be used within a ThemeProvider");return e}},8185:(e,s,l)=>{"use strict";l.d(s,{A:()=>r});var r=(0,l(6467).A)("outline","sun","IconSun",[["path",{d:"M12 12m-4 0a4 4 0 1 0 8 0a4 4 0 1 0 -8 0",key:"svg-0"}],["path",{d:"M3 12h1m8 -9v1m8 8h1m-9 8v1m-6.4 -15.4l.7 .7m12.1 -.7l-.7 .7m0 11.4l.7 .7m-12.1 -.7l-.7 .7",key:"svg-1"}]])}},e=>{var s=s=>e(e.s=s);e.O(0,[126,423,525,960,328,908,681,522,573,116,441,684,358],()=>s(5113)),_N_E=e.O()}]);