(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[974],{3792:(e,r,u)=>{"use strict";u.r(r),u.d(r,{default:()=>t});var s=u(5695);function t(){(0,s.redirect)("/dashboard")}},5695:(e,r,u)=>{"use strict";var s=u(8999);u.o(s,"redirect")&&u.d(r,{redirect:function(){return s.redirect}}),u.o(s,"useRouter")&&u.d(r,{useRouter:function(){return s.useRouter}}),u.o(s,"useSearchParams")&&u.d(r,{useSearchParams:function(){return s.useSearchParams}})},6146:(e,r,u)=>{Promise.resolve().then(u.bind(u,3792))}},e=>{var r=r=>e(e.s=r);e.O(0,[441,684,358],()=>r(6146)),_N_E=e.O()}]);