KEY_FILE: str = ".eval"
HIDDEN_DIR: str = ".eval"
PYTEST_RUN_TEST_NAME: str = "CONFIDENT_AI_RUN_TEST_NAME"
LOGIN_PROMPT = "\n✨👀 Looking for a place for your LLM test data to live 🏡❤️ ? Use [rgb(106,0,255)]Confident AI[/rgb(106,0,255)] to get & share testing reports, experiment with models/prompts, and catch regressions for your LLM system. Just run [cyan]'deepeval login'[/cyan] in the CLI."


CONFIDENT_TRACE_VERBOSE = "CONFIDENT_TRACE_VERBOSE"
CONFIDENT_TRACE_FLUSH = "CONFIDENT_TRACE_FLUSH"
CONFIDENT_SAMPLE_RATE = "CONFIDENT_SAMPLE_RATE"
CONFIDENT_TRACE_ENVIRONMENT = "CONFIDENT_TRACE_ENVIRONMENT"
